"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const quidproquo_webserver_1 = require("quidproquo-webserver");
const awsLambdaUtils_1 = require("../../../awsLambdaUtils");
const getProcessTransformEventParams = (appName) => {
    return (payload, session) => __awaiter(void 0, void 0, void 0, function* () {
        const [apiGatewayEvent, context] = payload.eventParams;
        const path = (apiGatewayEvent.path || '').replace(new RegExp(`^(\/${appName})/`), '/');
        return {
            path,
            query: Object.assign(Object.assign({}, (apiGatewayEvent.multiValueQueryStringParameters || {})), (apiGatewayEvent.queryStringParameters || {})),
            body: apiGatewayEvent.body ? JSON.parse(apiGatewayEvent.body) : undefined,
            headers: apiGatewayEvent.headers,
            method: apiGatewayEvent.httpMethod,
            correlation: context.awsRequestId,
        };
    });
};
const getProcessTransformResponseResult = (domainName) => {
    return (payload, session) => __awaiter(void 0, void 0, void 0, function* () {
        return ({
            statusCode: payload.response.result.statusCode,
            body: payload.response.result.body,
            headers: {
                'Content-Type': 'application/json',
                'Access-Control-Allow-Headers': '*',
                'Access-Control-Allow-Methods': '*',
                'Access-Control-Allow-Origin': `https://${domainName}`,
            },
        });
    });
};
const getProcessAutoRespond = (domainName) => {
    return (payload, session) => __awaiter(void 0, void 0, void 0, function* () {
        if (payload.transformedEventParams.method === 'OPTIONS') {
            return {
                statusCode: 200,
                headers: {
                    'Content-Type': 'application/json',
                    'Access-Control-Allow-Headers': '*',
                    'Access-Control-Allow-Methods': '*',
                    'Access-Control-Allow-Origin': `https://${domainName}`,
                },
            };
        }
        return null;
    });
};
const getProcessMatchStory = (routes) => (payload, session) => __awaiter(void 0, void 0, void 0, function* () {
    // Sort the routes by string length
    // Note: We may need to filter variable routes out {} as the variables are length independent
    const sortedRoutes = routes
        .filter((r) => r.method === payload.transformedEventParams.method)
        .sort((a, b) => {
        if (a.path.length < b.path.length)
            return -1;
        if (a.path.length > b.path.length)
            return 1;
        return 0;
    });
    // Find the most relevant match
    return sortedRoutes.find((r) => (0, awsLambdaUtils_1.matchUrl)(r.path, payload.transformedEventParams.path).didMatch);
});
exports.default = (config) => {
    const routes = quidproquo_webserver_1.qpqWebServerUtils.getAllRoutes(config);
    const appName = quidproquo_core_1.qpqCoreUtils.getAppName(config);
    const domainName = quidproquo_webserver_1.qpqWebServerUtils.getDomainName(config);
    return {
        [quidproquo_core_1.EventActionTypeEnum.TransformEventParams]: getProcessTransformEventParams(appName),
        [quidproquo_core_1.EventActionTypeEnum.TransformResponseResult]: getProcessTransformResponseResult(domainName),
        [quidproquo_core_1.EventActionTypeEnum.AutoRespond]: getProcessAutoRespond(domainName),
        [quidproquo_core_1.EventActionTypeEnum.MatchStory]: getProcessMatchStory(routes),
    };
};
